@@ -28,6 +28,7 @@
             (:constructor %make-dll-header))
 )
 
+
 (defmacro dll-header-last (h) `(dll-header-pred ,h))
 (defmacro dll-header-first (h) `(dll-header-succ ,h))
 
@@ -197,4 +198,26 @@
            ,@body)
          (return-dll-nodes ,internal-header-name ,internal-freelist-name)))))
 
-(ccl::provide "DLL-NODE")
+(defstruct (locked-dll-header
+	     (:include dll-header)
+	     (:constructor %make-locked-dll-header))
+  (lock (make-lock)))
+
+(defun make-locked-dll-header ()
+  (init-dll-header (%make-locked-dll-header)))
+
+(defmacro with-locked-dll-header ((h) &body body)
+  `(with-lock-grabbed ((locked-dll-header-lock ,h))
+    ,@body))
+
+(defun locked-dll-header-enqueue (node header)
+  (with-locked-dll-header (header)
+    (append-dll-node node header)))
+
+(defun locked-dll-header-dequeue (header)
+  (with-locked-dll-header (header)
+    (let* ((first (dll-header-first header)))
+      (unless (eq first header)
+	(remove-dll-node first)))))
+
+(provide "DLL-NODE")
