@@ -22,7 +22,7 @@
 
 (defvar *logical-register-counter* -1)
 
-(defvar *lreg-freelist* (%cons-pool))
+(def-standard-initial-binding *lreg-freelist* (%cons-pool))
 
 (defstruct (lreg
             (:print-function print-lreg)
@@ -92,7 +92,7 @@
 	    (format s "{?/~a}" mode-name)
 	    (format s "{?}")))))))
 
-(defvar *lcell-freelist* (%cons-pool))
+(def-standard-initial-binding *lcell-freelist* (%cons-pool))
 (defvar *next-lcell-id* -1)
 
 (defstruct (lcell 
@@ -189,6 +189,8 @@
     (:s16 . ,arch::storage-class-s16)
     (:u32 . ,arch::storage-class-u32)
     (:s32 . ,arch::storage-class-s32)
+    (:u64 . ,arch::storage-class-u64)
+    (:s64 . ,arch::storage-class-s64)
     (:address . ,arch::storage-class-address)
     (:single-float . ,arch::storage-class-single-float)
     (:double-float . ,arch::storage-class-double-float)
@@ -196,7 +198,7 @@
     (:locative . ,arch::storage-class-locative)
     (:crf . ,arch::storage-class-crf)
     (:crbit . ,arch::storage-class-crbit)
-    (:crfbit . ,arch::storage-class-crfbit)
+    (:crfbit . ,arch::storage-class-crfbit)   
     (t . nil)))
     
 (defun spec-class->storage-class (class-name)
