@@ -138,13 +138,13 @@
 
 (defun %fhave (name def)
   (let* ((fname (validate-function-name name)))
-    (setf (%svref (%symbol->symptr fname) arch::symbol.fcell-cell) def)))
+    (setf (%svref (%symbol->symptr fname) ppc32::symbol.fcell-cell) def)))
 
 ; FBOUNDP is true of any symbol whose function-cell contains something other
 ; than %unbound-function%; we expect FBOUNDP to return that something.
 (defun fboundp (name)
   (let* ((fname (validate-function-name name))
-         (def (%svref (%symbol->symptr fname) arch::symbol.fcell-cell)))
+         (def (%svref (%symbol->symptr fname) ppc32::symbol.fcell-cell)))
     (unless (eq def %unbound-function%)
       def)))
 
@@ -153,8 +153,8 @@
 
 (defun %unfhave (sym)
   (let* ((symptr (%symbol->symptr sym))
-         (old (%svref symptr arch::symbol.fcell-cell)))
-    (setf (%svref symptr arch::symbol.fcell-cell) %unbound-function%)
+         (old (%svref symptr ppc32::symbol.fcell-cell)))
+    (setf (%svref symptr ppc32::symbol.fcell-cell) %unbound-function%)
     (not (eq old %unbound-function%))))
 
 ; It's guaranteed that lfun-bits is a fixnum.  Might be a 30-bit fixnum ...
@@ -180,14 +180,21 @@
   fun)
 
 (defun lfun-vector-name (fun &optional (new-name nil set-name-p))
-  (let* ((has-name-cell (not (logbitp 29 (the fixnum (lfun-bits fun))))))
-    (if has-name-cell
-      (let* ((name-idx (- (the fixnum (uvsize fun)) 2))
-             (old-name (%svref fun name-idx)))
-        (declare (fixnum name-idx))
-        (if (and set-name-p (not (eq old-name new-name)))
-          (setf (%svref fun name-idx) new-name))
-        old-name))))
+  (let* ((bits (lfun-bits fun)))
+    (declare (fixnum bits))
+    (if (and (logbitp $lfbits-gfn-bit bits)
+	     (not (logbitp $lfbits-method-bit bits)))
+      (if set-name-p
+	(%gf-name fun new-name)
+	(%gf-name fun))
+      (let* ((has-name-cell (not (logbitp $lfbits-noname-bit bits))))
+	(if has-name-cell
+	  (let* ((name-idx (- (the fixnum (uvsize fun)) 2))
+		 (old-name (%svref fun name-idx)))
+	    (declare (fixnum name-idx))
+	    (if (and set-name-p (not (eq old-name new-name)))
+	      (setf (%svref fun name-idx) new-name))
+	    old-name))))))
 
 (defun lfun-name (fun &optional (new-name nil set-name-p))
   (multiple-value-bind (stored-name stored?) (lookup-lfun-name fun)
