@@ -26,6 +26,21 @@
 ;; This is called only by the compiler output of a PROGV form.
 ;; It checks for the maximum length that the progvsave subprim
 ;; can handle.
+(defun svar-check-symbol-list (l &optional (max-length
+                                        (floor (- 4096 20) (* 4 3))
+                                       ))
+  (let ((len (list-length l)))
+    (if (and len
+             (or (null max-length)
+                 (< len max-length))
+             (dolist (s l t) 
+               (unless (and (symbolp s)
+                            (not (constant-symbol-p s))
+                            (not (logbitp $sym_vbit_global (the fixnum (%symbol-bits s)))))
+                 (return nil))))
+      (mapcar #'ensure-svar l)
+      (error "~s is not a proper list of bindable symbols~@[ of length < ~s~]." l max-length))))
+
 (defun check-symbol-list (l &optional (max-length
                                         (floor (- 4096 20) (* 4 3))
                                        ))
@@ -34,7 +49,9 @@
              (or (null max-length)
                  (< len max-length))
              (dolist (s l t) 
-               (unless (and (symbolp s) (not (constant-symbol-p s)))
+               (unless (and (symbolp s)
+                            (not (constant-symbol-p s))
+                            (not (logbitp $sym_vbit_global (the fixnum (%symbol-bits s)))))
                  (return nil))))
       l
       (error "~s is not a proper list of bindable symbols~@[ of length < ~s~]." l max-length))))
@@ -129,7 +146,7 @@
            (constant-symbol-p sym))
     (cerror "Make ~S be unbound anyway."
             "~S is a constant; making it unbound might be a bad idea." sym))
-  (%symbol-bits sym (the fixnum (logand (logior #xff00 (%ilsl $sym_bit_special 1))
+  (%symbol-bits sym (the fixnum (logand (logior #xff00 (ash 1 $sym_bit_special) (ash 1 $sym_vbit_bound))
                                         (the fixnum (%symbol-bits sym)))))
   (%set-sym-value sym (%unbound-marker))
   sym)
@@ -146,7 +163,7 @@
   (not (eq (%sym-value sym) (%unbound-marker))))
 
 (defun make-symbol (name)
-  (%gvector arch::subtag-symbol
+  (%gvector ppc32::subtag-symbol
                 (require-type name 'simple-string) ; pname
                 (%unbound-marker)       ; value cell
                 %unbound-function%      ; function cell
@@ -155,27 +172,33 @@
 
 (defun %symbol-bits (sym &optional new)
   (let* ((p (%symbol->symptr sym))
-         (bits (%svref p arch::symbol.flags-cell)))
+         (bits (%svref p ppc32::symbol.flags-cell)))
     (if new
-      (setf (%svref p arch::symbol.flags-cell) new))
+      (setf (%svref p ppc32::symbol.flags-cell) new))
     bits))
 
 (defun %sym-value (name)
-  (%svref (%symbol->symptr name) arch::symbol.vcell-cell))
+  (%svar-sym-value (%ensure-svar (%symbol->symptr name))))
 
 (defun %set-sym-value (name val)
-  (setf (%svref (%symbol->symptr name) arch::symbol.vcell-cell) val))
+  (%svar-set-sym-value (%ensure-svar (%symbol->symptr name)) val))
+
+(defun %sym-global-value (name)
+  (%svref (%symbol->symptr name) ppc32::symbol.vcell-cell))
+
+(defun %set-sym-global-value (name val)
+  (setf (%svref (%symbol->symptr name) ppc32::symbol.vcell-cell) val))
 
 (defun %symbol-package-plist (sym)
-  (%svref (%symbol->symptr sym) arch::symbol.package-plist-cell))
+  (%svref (%symbol->symptr sym) ppc32::symbol.package-plist-cell))
 
 (defun %set-symbol-package-plist (sym new)
-  (setf (%svref (%symbol->symptr sym) arch::symbol.package-plist-cell) new))
+  (setf (%svref (%symbol->symptr sym) ppc32::symbol.package-plist-cell) new))
 
 
 
 (defun symbol-name (sym)
-  (%svref (%symbol->symptr sym) arch::symbol.pname-cell))
+  (%svref (%symbol->symptr sym) ppc32::symbol.pname-cell))
 
 
 
@@ -186,3 +209,52 @@
              (= (the fixnum (uvsize fbinding)) 2))
       (let* ((fun (%svref fbinding 1)))
         (if (functionp fun) fun)))))
+
+(defun %symbol-binding-address (sym)
+  (%svar-binding-address (ensure-svar sym)))
+
+(let* ((svar-lock (make-lock))
+       (svar-hash (make-hash-table :test #'eq :weak t))
+       (svar-idx-map (make-hash-table :test #'eq :weak :value))
+       (svar-index 0))
+  (defun %set-svar-hash (hash)
+    (unless svar-hash
+      (setq svar-hash hash)))
+  (defun %find-svar (symptr)
+    (gethash symptr svar-hash))
+  (defun find-svar (sym)
+    (%find-svar (%symbol->symptr sym)))
+  (defun ensure-svar-idx (svar)
+    (let* ((sym (%svref svar ppc32::svar.symbol-cell))
+           (idx (%svref svar ppc32::svar.idx-cell))
+           (bits (%symbol-bits sym)))
+      (declare (fixnum idx bits))
+      (if (or (logbitp $sym_vbit_global bits)
+              (logbitp $sym_vbit_const bits))
+        (unless (zerop idx)
+	  (remhash idx svar-idx-map)
+          (setf (%svref svar ppc32::svar.idx-cell) 0))
+        (if (zerop idx)
+	  (let* ((new-idx (incf svar-index)))
+	    (setf (%svref svar ppc32::svar.idx-cell) new-idx)
+	    (setf (gethash new-idx svar-idx-map) svar))))
+      svar))
+  (defun %ensure-svar (symptr)
+    (with-lock-grabbed (svar-lock)
+      (let* ((svar (or (%find-svar symptr)
+                       (setf (gethash symptr svar-hash)
+                             (gvector :svar symptr 0)))))
+        (ensure-svar-idx svar))))
+  (defun ensure-svar (sym)
+    (%ensure-svar (%symbol->symptr sym)))
+  (defun cold-load-svar (svar)
+    (with-lock-grabbed (svar-lock)
+      (let* ((symptr (%symbol->symptr (%svref svar ppc32::svar.symbol-cell))))
+        (setf (gethash symptr svar-hash) svar)
+        (setf (%svref svar ppc32::svar.idx-cell) 0)
+        (ensure-svar-idx svar))))
+  (defun svar-index ()
+    svar-index)
+  (defun index-svar (index)
+    (gethash index svar-idx-map))
+  )
