@@ -20,7 +20,27 @@
 (in-package :ccl)
 
 
+(let* ((standard-initial-bindings ())
+       (standard-initial-bindings-lock (make-read-write-lock)))
 
+  (defun standard-initial-bindings ()
+    (with-read-lock (standard-initial-bindings-lock)
+      (copy-list standard-initial-bindings)))
+
+  (defun define-standard-initial-binding (symbol initform)
+    (setq symbol (require-type symbol 'symbol))
+    (%proclaim-special symbol)
+    (unless (boundp symbol)
+      (set symbol (funcall initform)))
+    (with-write-lock (standard-initial-bindings-lock)
+      (let* ((pair (assoc symbol standard-initial-bindings)))
+	(if pair
+	  (setf (cdr pair) initform)
+	  (push (cons symbol initform) standard-initial-bindings))))
+    (record-source-file symbol 'variable)
+    symbol))
+
+(def-standard-initial-binding *package*)
 
 (defun %badarg (arg type)
   (%err-disp $XWRONGTYPE arg type))
@@ -116,37 +136,53 @@
           ((atom x) (unless (null x)
                       (%rplacd splice x)) result)))))
 
+(defun alt-list-length (l)
+  "Detect (and complain about) cirucular lists; allow any atom to
+terminate the list"
+  (do* ((n 0 (1+ n))
+        (fast l)
+        (slow l))
+       ((atom fast) n)
+    (declare (fixnum n))
+    (setq fast (cdr fast))
+    (if (logbitp 0 n)
+      (if (eq (setq slow (cdr slow)) fast)
+	(%err-disp $XIMPROPERLIST l)))))
 
-; take two args this week
 
 (defun last (list &optional (n 1))
-  (unless (and (typep n 'fixnum)
-               (>= (the fixnum n) 0))
-    (report-bad-arg n '(and fixnum unsigned-byte)))
-  (locally (declare (fixnum n))
-    (do* ((checked-list list (cdr checked-list))
-          (returned-list list)
-          (index 0 (1+ index)))
-         ((atom checked-list) returned-list)
-      (declare (type index index))
-      (if (>= index n)
-	  (pop returned-list)))))
+  (if (and (typep n 'fixnum)
+	   (>= (the fixnum n) 0))
+    (locally (declare (fixnum n))
+      (do* ((checked-list list (cdr checked-list))
+	    (returned-list list)
+	    (index 0 (1+ index)))
+	   ((atom checked-list) returned-list)
+	(declare (type index index))
+	(if (>= index n)
+	  (pop returned-list))))
+    (if (and (typep n 'bignum)
+	     (> n 0))
+      (require-type list 'list)
+      (report-bad-arg  n 'unsigned-byte))))
 
 
 
 
 
 (defun nthcdr (index list)
-  (unless (and (typep index 'fixnum)
-               (>= (the fixnum index) 0))
-    (report-bad-arg index '(and fixnum unsigned-byte)))
-  (locally (declare (fixnum index))
-    (do* ()
-         ((< (decf index) 0))
-      (when (null (setq list (cdr list))) (return)))
-    list))
-
-
+  (if (and (typep index 'fixnum)
+	   (>= (the fixnum index) 0))
+    (locally (declare (fixnum index))
+      (dotimes (i index list)
+	(when (null (setq list (cdr list))) (return))))
+    (progn
+      (unless (typep index 'unsigned-byte)
+	(report-bad-arg index 'unsigned-byte))
+      (do* ((n index (- n most-positive-fixnum)))
+	   ((typep n 'fixnum) (nthcdr n list))
+	(unless (setq list (nthcdr most-positive-fixnum list))
+	  (return))))))
 
 
 (defun nth (index list) (car (nthcdr index list)))
@@ -223,14 +259,6 @@
     (if valid-p nm (error "Invalid function name ~s." name))))
 
 
-
-
-
-#-macos
-(defun real-xstring-p (x) 
-  (declare (ignore x))
-  nil)
-
 ; Returns index if char appears in string, else nil.
 
 (defun %str-member (char string &optional start end)
@@ -255,10 +283,7 @@
   (dotimes (i (the fixnum (length vector)))
     (when (eq elt (%svref vector i)) (return i))))
 
-
-
-
-
+(defun logical-pathname-p (thing) (istruct-typep thing 'logical-pathname))
 
 (progn
 ; It's back ...
@@ -284,18 +309,14 @@
    (vector-nreverse seq)))
 )
 
-
-
 (defun nreconc (x y)
   "Returns (nconc (nreverse x) y)"
-  (do ((1st (cdr x) (if (atom 1st) 1st (cdr 1st)))
+  (do ((1st (cdr x) (if (endp 1st) 1st (cdr 1st)))
        (2nd x 1st)		;2nd follows first down the list.
        (3rd y 2nd))		;3rd follows 2nd down the list.
       ((atom 2nd) 3rd)
     (rplacd 2nd 3rd)))
 
-
-
 (defun append (&lexpr lists)
   (let* ((n (%lexpr-count lists)))
     (declare (fixnum n))
@@ -335,22 +356,23 @@
   (seq-dispatch seq (list-reverse seq) (vector-reverse seq)))
 )
 
-(defun check-sequence-bounds (seq start end &optional length)
-  (if (not length) 
-    (setq length (length seq)))
-  (if (not end)
-    (setq end length)
-    (unless (typep end 'fixnum)
-      (report-bad-arg end 'fixnum)))
-  (unless (typep start 'fixnum)
-    (report-bad-arg start 'fixnum))
-  (cond ((> end length)
-         (report-bad-arg end `(integer 0 (,length))))
-	((< start 0)
-	 (report-bad-arg start `(integer 0)))
-        ((> start end)
-         (report-bad-arg start `(integer 0 ,end)))
-        (t end)))
+(defun check-sequence-bounds (seq start end)
+  (let* ((length (length seq)))
+    (declare (fixnum length))
+    (if (not end)
+      (setq end length)
+      (unless (typep end 'fixnum)
+	(report-bad-arg end 'fixnum)))
+    (unless (typep start 'fixnum)
+      (report-bad-arg start 'fixnum))
+    (locally (declare (fixnum start end))
+      (cond ((> end length)
+	     (report-bad-arg end `(integer 0 (,length))))
+	    ((< start 0)
+	     (report-bad-arg start `(integer 0)))
+	    ((> start end)
+	     (report-bad-arg start `(integer 0 ,end)))
+	    (t end)))))
   
 
 (defun byte-length (string &optional  (start 0) end)
@@ -422,13 +444,13 @@
 
 (defun coerce-to-uvector (object subtype simple-p)  ; simple-p ?  
   (let ((type-code (typecode object)))
-    (cond ((eq type-code arch::tag-list)
+    (cond ((eq type-code target::tag-list)
            (%list-to-uvector subtype object))
-          ((>= type-code arch::min-cl-ivector-subtag)  ; 175
+          ((>= type-code target::min-cl-ivector-subtag)  ; 175
            (if (or (null subtype)(= subtype type-code))
              (return-from coerce-to-uvector object)))
-          ((>= type-code arch::min-vector-subtag)     ; 170
-           (if (= type-code arch::subtag-simple-vector)
+          ((>= type-code target::min-vector-subtag)     ; 170
+           (if (= type-code target::subtag-simple-vector)
              (if (or (null subtype)
                      (= type-code subtype))
                (return-from coerce-to-uvector object))
@@ -437,7 +459,7 @@
                           (= subtype (typecode (array-data-and-offset object)))))
                (return-from coerce-to-uvector object))))
           (t (error "Can't coerce ~s to Uvector" object))) ; or just let length error
-    (if (null subtype)(setq subtype arch::subtag-simple-vector))
+    (if (null subtype)(setq subtype target::subtag-simple-vector))
     (let* ((size (length object))
            (val (%alloc-misc size subtype)))
       (declare (fixnum size))
@@ -457,7 +479,7 @@
 ; 3 callers
 (defun %list-to-uvector (subtype list)   ; subtype may be nil (meaning simple-vector
   (let* ((n (length list))
-         (new (%alloc-misc n (or subtype arch::subtag-simple-vector))))  ; yech
+         (new (%alloc-misc n (or subtype target::subtag-simple-vector))))  ; yech
     (dotimes (i n)
       (declare (fixnum i))
       (uvset new i (%car list))
@@ -495,15 +517,13 @@
   ; given uvector subtype - what is the corresponding element-type
   (defun element-subtype-type (subtype)
     (declare (fixnum subtype))
-    (if  (= subtype arch::subtag-simple-vector) t
+    (if  (= subtype ppc32::subtag-simple-vector) t
         (svref array-element-subtypes 
-               (ash (- subtype arch::min-cl-ivector-subtag) (- arch::ntagbits)))))
+               (ash (- subtype ppc32::min-cl-ivector-subtag) (- ppc32::ntagbits)))))
   )
 
 
-(defun %make-temp-uvector (len &optional subtype)
-  ; that will be wrong too 
-  (%alloc-misc len (or subtype arch::subtag-simple-vector)))
+
 
 
 
@@ -516,14 +536,14 @@
 ; %make-displaced-array assumes the following
 
 (eval-when (:compile-toplevel)
-  (assert (eql arch::arrayH.flags-cell arch::vectorH.flags-cell))
-  (assert (eql arch::arrayH.displacement-cell arch::vectorH.displacement-cell))
-  (assert (eql arch::arrayH.data-vector-cell arch::vectorH.data-vector-cell)))
+  (assert (eql target::arrayH.flags-cell target::vectorH.flags-cell))
+  (assert (eql target::arrayH.displacement-cell target::vectorH.displacement-cell))
+  (assert (eql target::arrayH.data-vector-cell target::vectorH.data-vector-cell)))
 
 
 (defun %make-displaced-array (dimensions displaced-to
-                                         &optional fill adjustable offset temp-p)
-  (declare (ignore temp-p))
+                                         &optional fill adjustable
+					 offset explicitp)
   (if offset 
     (unless (and (fixnump offset) (>= (the fixnum offset) 0))
       (setq offset (require-type offset '(and fixnum (integer 0 *)))))
@@ -543,6 +563,8 @@
            (real-offset offset)
            (flags 0))
       (declare (fixnum disp-size rank flags vect-subtype real-offset))
+      (when explicitp
+	(setq flags (bitset $arh_exp_disp_bit flags)))
       (if (not (fixnump new-size))(error "Bad array dimensions ~s." dimensions)) 
       (locally (declare (fixnum new-size))
         ; (when (> (+ offset new-size) disp-size) ...), but don't cons bignums
@@ -565,39 +587,60 @@
       ; adjustable or its target is a header, then we need to set the
       ; $arh_disp_bit. If displaced-to is not adjustable, then our
       ; target can be its target instead of itself.
-      (when (or (eql vect-subtype arch::subtag-arrayH)
-                (eql vect-subtype arch::subtag-vectorH))
-        (let ((dflags (%svref displaced-to arch::arrayH.flags-cell)))
+      (when (or (eql vect-subtype target::subtag-arrayH)
+                (eql vect-subtype target::subtag-vectorH))
+        (let ((dflags (%svref displaced-to target::arrayH.flags-cell)))
           (declare (fixnum dflags))
           (when (or (logbitp $arh_adjp_bit dflags)
+		    t
                     (progn
-                      (setq target (%svref displaced-to arch::arrayH.data-vector-cell)
-                            real-offset (+ offset (%svref displaced-to arch::arrayH.displacement-cell)))
-                      (logbitp $arh_disp_bit dflags)))
+		      #+nope
+                      (setq target (%svref displaced-to target::arrayH.data-vector-cell)
+                            real-offset (+ offset (%svref displaced-to target::arrayH.displacement-cell)))
+                      (logbitp $arh_disp_bit dflags)
+		      #-nope t))
             (setq flags (bitset $arh_disp_bit flags))))
         (setq vect-subtype (%array-header-subtype displaced-to)))
       ; assumes flags is low byte
-      (setq flags (dpb vect-subtype arch::arrayH.flags-cell-subtag-byte flags))
+      (setq flags (dpb vect-subtype target::arrayH.flags-cell-subtag-byte flags))
       (if (eq rank 1)
-        (%gvector arch::subtag-vectorH 
+        (%gvector target::subtag-vectorH 
                       (if (fixnump fill) fill new-size)
                       new-size
                       target
                       real-offset
                       flags)
-        (let ((val (%alloc-misc (+ arch::arrayh.dim0-cell rank) arch::subtag-arrayH)))
-          (setf (%svref val arch::arrayH.rank-cell) rank)
-          (setf (%svref val arch::arrayH.physsize-cell) new-size)
-          (setf (%svref val arch::arrayH.data-vector-cell) target)
-          (setf (%svref val arch::arrayH.displacement-cell) real-offset)
-          (setf (%svref val arch::arrayH.flags-cell) flags)
+        (let ((val (%alloc-misc (+ target::arrayh.dim0-cell rank) target::subtag-arrayH)))
+          (setf (%svref val target::arrayH.rank-cell) rank)
+          (setf (%svref val target::arrayH.physsize-cell) new-size)
+          (setf (%svref val target::arrayH.data-vector-cell) target)
+          (setf (%svref val target::arrayH.displacement-cell) real-offset)
+          (setf (%svref val target::arrayH.flags-cell) flags)
           (do* ((dims dimensions (cdr dims))
                 (i 0 (1+ i)))              
                ((null dims))
             (declare (fixnum i)(list dims))
-            (setf (%svref val (%i+ arch::arrayH.dim0-cell i)) (car dims)))
+            (setf (%svref val (%i+ target::arrayH.dim0-cell i)) (car dims)))
           val)))))
 
+(defun make-array (dims &key (element-type t element-type-p)
+                        displaced-to
+                        displaced-index-offset
+                        adjustable
+                        fill-pointer
+                        (initial-element nil initial-element-p)
+                        (initial-contents nil initial-contents-p))
+  (when (and initial-element-p initial-contents-p)
+        (error "Cannot specify both ~S and ~S" :initial-element-p :initial-contents-p))
+  (make-array-1 dims element-type element-type-p
+                displaced-to
+                displaced-index-offset
+                adjustable
+                fill-pointer
+                initial-element initial-element-p
+                initial-contents initial-contents-p
+                nil))
+
 
 
 
@@ -673,31 +716,17 @@
 
 
 
-
+(defvar *periodic-task-interval* 0.3)
+(defvar *periodic-task-seconds* 0)
+(defvar *periodic-task-nanoseconds* 300000000)
 
 (defun set-periodic-task-interval (n)
-  n)
+  (multiple-value-setq (*periodic-task-seconds* *periodic-task-nanoseconds*)
+    (nanoseconds n))
+  (setq *periodic-task-interval* n))
 
 (defun periodic-task-interval ()
-  1)
-
-
-
-(defparameter char-up-string-1
-  " 	
-
- !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`ABCDEFGHIJKLMNOPQRSTUVWXYZ{|}~́󆠡I")
-
- 
-(defparameter char-down-string-1
-  " 	
-
- !\"#$%&'()*+,-./0123456789:;<=>?@abcdefghijklmnopqrstuvwxyz[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~ʈ䉐𘜞")
- 
-
-
-; for chars between #\200 and #\237
-
+  *periodic-task-interval*)
 
 
 
@@ -758,6 +787,16 @@
       (multiple-value-bind (str off)(array-data-and-offset string)
         (values str (%i+ off start)(%i+ off end))))))
 
+(defun get-properties (place indicator-list)
+  "Like GETF, except that Indicator-List is a list of indicators which will
+  be looked for in the property list stored in Place.  Three values are
+  returned, see manual for details."
+  (do ((plist place (cddr plist)))
+      ((null plist) (values nil nil nil))
+    (cond ((atom (cdr plist))
+	   (report-bad-arg place '(satisfies plistp)))
+	  ((memq (car plist) indicator-list) ;memq defined in kernel
+	   (return (values (car plist) (cadr plist) plist))))))
 
 (defun string= (string1 string2 &key start1 end1 start2 end2)
     (locally (declare (optimize (speed 3)(safety 0)))
@@ -770,92 +809,14 @@
       (%simple-string= string1 string2 start1 start2 end1 end2)))
 
 
-; redefined in chars 
-(defun string-equal (string1 string2 &key start1 end1 start2 end2)
-  (multiple-value-setq (string1 start1 end1)(string-start-end string1 start1 end1))
-  (multiple-value-setq (string2 start2 end2)(string-start-end string2 start2 end2))
-  (when (eq (%i- end2 start2) (%i- end1 start1))
-    (let ((upstring char-up-string-1))
-      (declare (type (simple-array (unsigned-byte 8) (*)) upstring)
-               (optimize (speed 3)(safety 0)))          ; believe the lie.
-      (do* ((i start1 (%i+ 1 i))
-            (j start2 (%i+ 1 j)))
-           ((%i>= j end2))
-        (let* ((c1 (aref upstring (%scharcode string1 i)))
-               (c2 (aref upstring (%scharcode string2 j))))
-          (when (neq c1 c2)(return-from string-equal nil))))
-      t)))
-        
-; ditto
-(defun string-lessp (string1 string2 &key start1 end1 start2 end2)
-  (multiple-value-setq (string1 start1 end1)(string-start-end string1 start1 end1))
-  (multiple-value-setq (string2 start2 end2)(string-start-end string2 start2 end2))
-    (let ((upstring char-up-string-1))
-      (declare (type (simple-array (unsigned-byte 8) (*)) upstring)
-               (optimize (speed 3)(safety 0)))          ; stop this nonsense, please!.
-      (do* ((i start1 (%i+ 1 i))
-            (j start2 (%i+ 1 j)))
-           ((or (eq j end2)(eq i end1))(if (< j end2) (- i start1) nil))
-        (let* ((c1 (aref upstring (%scharcode string1 i)))
-               (c2 (aref upstring (%scharcode string2 j))))
-          (when (neq c1 c2)
-            (if (> c1 c2)
-              (return-from string-lessp nil)
-              (return-from string-lessp (- i start1))))))))
-
-
-
-  
-
-
-
-
-
-
-
-(defun lfun-attributes (lfun &optional new-value)
-  (declare (ignore lfun new-value))
-  0)
-
-(progn
-  ; only returns 1 value
-  (defun %nth-immediate (lfv i &optional (n (%count-immrefs lfv)))
-    (declare (fixnum i n))
-    (unless (and (>= i 0) (< i n))
-      (report-bad-arg i `(integer 0 ,n)))
-    (%svref lfv (the fixnum (1+ i))))
-  
-  (defun %count-immrefs (lfv)
-    (- (uvsize lfv) 2)))
-
-; No big deal about NILREG-[FV]CELL-SYMBOLS anymore, so last value always false.
-
-
 (defun lfun-keyvect (lfun)
-  ;Don't bother with kernel fns, they're going away.
-  (let ((lfv (%lfun-vector lfun)))
-    (when lfv
-      (let ((bits (lfun-bits lfun)))
-        (declare (fixnum bits))
-        (and (logbitp $lfbits-keys-bit bits)
-             (or (logbitp $lfbits-method-bit bits)
-                 (and (not (logbitp $lfbits-gfn-bit bits))
-                      (not (logbitp $lfbits-cm-bit bits))))
-             (if  (typep lfv 'interpreted-function) ; patch needs interpreted-method-function too
-
-               (nth 4 (evalenv-fnentry (%nth-immediate lfv 0))) ; gag puke
-               (%nth-immediate lfv 0)))))))
-
-
-(progn  ; ???
-
-(defun %lfun-vector-lfun (lfv) lfv)
-
-(defun %lfun-vector (lfn &optional load-p)      ; this is in level-0;68k-def
-  (declare (ignore load-p))
-  lfn)
-
-)
+  (let ((bits (lfun-bits lfun)))
+    (declare (fixnum bits))
+    (and (logbitp $lfbits-keys-bit bits)
+         (or (logbitp $lfbits-method-bit bits)
+             (and (not (logbitp $lfbits-gfn-bit bits))
+                  (not (logbitp $lfbits-cm-bit bits))))
+         (%svref lfun 1))))
 
 
 
@@ -864,7 +825,7 @@
   (let* ((bits (lfun-bits (setq fn (require-type fn 'function)))))
     (declare (fixnum bits))
     (if (logbitp $lfbits-trampoline-bit bits)
-      (function-lambda-expression (%nth-immediate (%lfun-vector fn) 0))
+      (function-lambda-expression (%svref fn 1))
       (values (uncompile-function fn)
               (logbitp $lfbits-nonnullenv-bit bits)
               (function-name fn)))))
@@ -895,61 +856,19 @@
   object)
 
 
-            
-
-
-(unless (fboundp 'call-toplevel-function)
-
-(defun call-toplevel-function (fun)
-  (funcall fun))
-
-) ; end unless
-
 
 (defun %set-toplevel (&optional (fun nil fun-p))
   ;(setq fun (require-type fun '(or symbol function)))
-  (if (eq *current-process* *initial-process*)
-    (prog1
-      %toplevel-function%
+  (let* ((tcr (%current-tcr)))
+    (prog1 (%tcr-toplevel-function tcr)
       (when fun-p
-        (setq %toplevel-function% fun)))
-    (let* ((p *current-process*)
-           (function.args (process.initial-form p)))
-      (prog1
-        (if (eq (car function.args) #'call-toplevel-function)
-          (cadr function.args)
-          (if (cdr function.args)
-            (let ((function.args (copy-list function.args)))
-              #'(lambda () (apply (car function.args) (cdr function.args))))
-            (car function.args)))
-        (when fun-p
-          (without-interrupts
-           (setf (car function.args) #'call-toplevel-function
-                 (cdr function.args) (list fun))))))))
+	(%set-tcr-toplevel-function tcr fun)))))
 
 ; Look! GC in Lisp !
 
 
-#+ppc-target
-(defppclapfunction full-gccount ()
-  (ref-global arg_z tenured-area)
-  (cmpwi cr0 arg_z 0)
-  (if :eq
-    (ref-global arg_z gc-count)
-    (lwz arg_z arch::area.gc-count arg_z))
-  (blr))
 
-#+sparc-target
-(defsparclapfunction full-gccount ()
-  (ref-global %arg_z tenured-area)
-  (tst %arg_z)
-  (bne done)
-   (nop)
-  (ref-global %arg_z gc-count)
-  (ld (%arg_z arch::area.gc-count) %arg_z)
-  done
-  (retl)
-   (nop))
+
   
 
 (defun gccounts ()
@@ -959,141 +878,72 @@
          (g1-count 0)
          (g0-count 0))
     (when (egc-enabled-p)
-      (let* ((a (%normalize-areas)))
-        (setq g0-count (%fixnum-ref a arch::area.gc-count) a (%fixnum-ref a arch::area.older))
-        (setq g1-count (%fixnum-ref a arch::area.gc-count) a (%fixnum-ref a arch::area.older))
-        (setq g2-count (%fixnum-ref a arch::area.gc-count))))
+      (let* ((a (%active-dynamic-area)))
+        (setq g0-count (%fixnum-ref a target::area.gc-count) a (%fixnum-ref a target::area.older))
+        (setq g1-count (%fixnum-ref a target::area.gc-count) a (%fixnum-ref a target::area.older))
+        (setq g2-count (%fixnum-ref a target::area.gc-count))))
     (values total full g2-count g1-count g0-count)))
 
       
-#+ppc-target
-(defppclapfunction gc ()
-  (uuo_xalloc rzero rzero rzero)
-  (mr arg_z rnil)
-  (blr))
 
-#+sparc-target
-(defsparclapfunction gc ()
-  (uuo_xalloc %rzero %rzero %rzero)
-  (retl)
-    (mov %rnil %arg_z))
 
-#+ppc-target
-(defppclapfunction purify ()
-  (uuo_xalloc rzero rnil rzero)
-  (mr arg_z rnil)
-  (blr))
 
-#+sparc-target
-(defsparclapfunction purify ()
-  (uuo_xalloc %rzero %rnil %rzero)
-  (retl)
-    (mov %rnil %arg_z))
-
-#+ppc-target
-(defppclapfunction lisp-heap-gc-threshold ()
-  (check-nargs 0)
-  (uuo_xalloc rnil rnil arg_z)
-  (blr))
-
-#+ppc-target
-(defppclapfunction set-lisp-heap-gc-threshold ((new arg_z))
-  (check-nargs 1)
-  (uuo_xalloc rnil new arg_z)
-  (blr))
-
-#+ppc-target
-(defppclapfunction use-lisp-heap-gc-threshold ()
-  (check-nargs 0)
-  (uuo_xalloc rnil rnil rnil)
-  (mr arg_z rnil)
-  (blr))
-
-;;;Procedure for creating pascal callable functions
-
-(defparameter %pascal-functions%
+(defglobal %pascal-functions%
   (make-array 4 :initial-element nil))
 
 
-; If name is already bound to a pascal-callable function, reuse the slot it occupies
-; so that pointers buried in Mac data structures remain valid.
+(defun gc-retain-pages (arg)
+  (setq *gc-event-status-bits*
+        (if arg
+          (bitset $gc-retain-pages-bit *gc-event-status-bits*)
+          (bitclr $gc-retain-pages-bit *gc-event-status-bits*)))
+  (not (null arg)))
+
+(defun gc-retaining-pages ()
+  (logbitp $gc-retain-pages-bit *gc-event-status-bits*))  
 
 
 
-
-
-
-; Called by kill-lisp-pointers
-(defun defpascal-trampoline-without-interrupts-p (code)
-  (and (macptrp code) (not (%null-ptr-p code)) (neq 0 (%get-word code 6))))
-
-
-
-(defun egc (arg)
-  (not (eql 0 (the fixnum (ff-call 
-                           (%kernel-import arch::kernel-import-egc-control) 
-                           :unsigned-halfword (if arg 1 0)
-                           :unsigned-halfword)))))
-
 (defun egc-active-p ()
   (and (egc-enabled-p)
        (not (eql 0 (%get-kernel-global 'oldest-ephemeral)))))
 
 ; this IS effectively a passive way of inquiring about enabled status.
 (defun egc-enabled-p ()
-  (not (eql 0 (%fixnum-ref (%normalize-areas) arch::area.older))))
+  (not (eql 0 (%fixnum-ref (%active-dynamic-area) target::area.older))))
 
 (defun egc-configuration ()
-  (let* ((g0 (%normalize-areas))
-         (g1 (%fixnum-ref g0 arch::area.older))
-         (g2 (%fixnum-ref g1 arch::area.older)))
-    (values (ash (the fixnum (%fixnum-ref g0 arch::area.threshold)) -8)
-            (ash (the fixnum (%fixnum-ref g1 arch::area.threshold)) -8)
-            (ash (the fixnum (%fixnum-ref g2 arch::area.threshold)) -8))))
+  (let* ((ta (%get-kernel-global 'tenured-area))
+         (g2 (%fixnum-ref ta target::area.younger))
+         (g1 (%fixnum-ref g2 target::area.younger))
+         (g0 (%fixnum-ref g1 target::area.younger)))
+    (values (ash (the fixnum (%fixnum-ref g0 target::area.threshold)) -8)
+            (ash (the fixnum (%fixnum-ref g1 target::area.threshold)) -8)
+            (ash (the fixnum (%fixnum-ref g2 target::area.threshold)) -8))))
 
 
 (defun configure-egc (e0size e1size e2size)
   (unless (egc-active-p)
-    (setq e2size (logand (lognot #x7fff) (+ #x7fff (ash (require-type e2size '(unsigned-byte 18)) 10)))
-          e1size (logand (lognot #x7fff) (+ #x7fff (ash (require-type e1size '(unsigned-byte 18)) 10)))
-          e0size (logand (lognot #x7fff) (+ #x7fff (ash (require-type e0size '(integer 1 #.(ash 1 18))) 10))))
-    (let* ((g0 (%normalize-areas))
-           (g1 (%fixnum-ref g0 arch::area.older))
-           (g2 (%fixnum-ref g1 arch::area.older)))
-      (%fixnum-set g0 arch::area.threshold (ash e0size (- arch::fixnumshift)))
-      (%fixnum-set g1 arch::area.threshold (ash e1size (- arch::fixnumshift)))
-      (%fixnum-set g2 arch::area.threshold (ash e2size (- arch::fixnumshift)))
-      t)))
+    (setq e2size (logand (lognot #xffff) (+ #xffff (ash (require-type e2size '(unsigned-byte 18)) 10)))
+          e1size (logand (lognot #xffff) (+ #xffff (ash (require-type e1size '(unsigned-byte 18)) 10)))
+          e0size (logand (lognot #xffff) (+ #xffff (ash (require-type e0size '(integer 1 #.(ash 1 18))) 10))))
+    (%configure-egc e0size e1size e2size)))
 
 
 
-
-; The question doesn't make sense for the PPC garbage collector,
-; but EGC is a good idea there, so return true
-(defun egc-mmu-support-available-p ()
-  t)
-
-
 (defun macptr-flags (macptr)
   (if (eql (uvsize (setq macptr (require-type macptr 'macptr))) 1)
     0
-    (uvref macptr ARCH::XMACPTR.FLAGS-CELL)))
+    (uvref macptr TARGET::XMACPTR.FLAGS-CELL)))
 
-#+ppc-target
-(defppclapfunction set-%gcable-macptrs% ((ptr ppc::arg_z))
-  (ref-global arg_y gcable-pointers)
-  (stw arg_y arch::xmacptr.link ptr)
-  (set-global ptr gcable-pointers)
-  (blr))
-
-(defun make-gcable-macptr (flags)
-  (let ((v (%alloc-misc arch::xmacptr.element-count arch::subtag-macptr)))
-    (setf (uvref v arch::xmacptr.address-cell) 0) ; ?? yup.
-    (setf (uvref v arch::xmacptr.flags-cell) flags)
-    (without-interrupts
-     (set-%gcable-macptrs% v))
-    v))
 
+; This doesn't really make the macptr be gcable (now has to be
+; on linked list), but we might have other reasons for setting
+; other flag bits.
+(defun set-macptr-flags (macptr value) 
+  (unless (eql (uvsize (setq macptr (require-type macptr 'macptr))) 1)
+    (setf (%svref macptr TARGET::XMACPTR.FLAGS-CELL) value)
+    value))
 
 (defun %new-gcable-ptr (size &optional clear-p)
   (let ((p (make-gcable-macptr $flags_DisposPtr)))
@@ -1102,55 +952,7 @@
       (#_bzero p size))
     p))
 
-
-
-
-
-; This doesn't really make the macptr be gcable (now has to be
-; on linked list), but we might have other reasons for setting
-; other flag bits.
-(defun set-macptr-flags (macptr value) 
-  (unless (eql (uvsize (setq macptr (require-type macptr 'macptr))) 1)
-    (setf (%svref macptr ARCH::XMACPTR.FLAGS-CELL) value)
-    value))
-
-(defun gc-event-check-enabled-p ()
-  (declare (special *gc-event-status-bits*))
-  (%i>= *gc-event-status-bits* 0))
-
-
-(defun set-gc-event-check-enabled-p (flag)
-  (declare (special *gc-event-status-bits*)
-           (fixnum  *gc-event-status-bits*))
-  ; Polling is disabled when $gc-polling-enabled-bit is set.
-  (setq *gc-event-status-bits*
-        (if flag 
-          (logand (lognot (ash -1 $gc-polling-enabled-bit)) *gc-event-status-bits*)
-          (logior (ash -1 $gc-polling-enabled-bit) *gc-event-status-bits*)))
-  flag)
-
-(defun gc-cursor-suppressed-p ()
-  (declare (special *gc-event-status-bits*)
-           (fixnum *gc-event-status-bits*))
-  (locally (declare (optimize (speed 3) (safety 0)))
-    (logbitp $GC-USE-GC-CURSOR-BIT *gc-event-status-bits*)))
-
-(defun set-gc-cursor-suppressed-p (flag)
-  (declare (special *gc-event-status-bits*)
-           (fixnum *gc-event-status-bits*))
-  (setq *gc-event-status-bits*
-        (locally (declare (optimize (speed 3) (safety 0)))      ; no $sp-specref
-          (if flag
-            (bitset $GC-USE-GC-CURSOR-BIT *gc-event-status-bits*)
-            (bitclr $GC-USE-GC-CURSOR-BIT *gc-event-status-bits*))))
-  flag)
-
-; backwards compat.
-(setf (symbol-function 'gc-cursor-supressed-p)     #'gc-cursor-suppressed-p
-      (symbol-function 'set-gc-cursor-supressed-p) #'set-gc-cursor-suppressed-p)
-
-
-;True for a-z - redefined in chars.lisp
+;True for a-z.
 (defun lower-case-p (c)
   (let ((code (char-code c)))
     (and (>= code (char-code #\a))
@@ -1166,6 +968,8 @@
         (and (>= code (char-code #\a)) (<= code (char-code #\z))))))
 
 
+
+
 ; def-accessors type-tracking stuff.  Used by inspector
 (defvar *def-accessor-types* nil)
 
@@ -1176,6 +980,98 @@
       (setf (cdr cell) (if (vectorp names) names (%list-to-uvector nil names))))))
 
 
+;;; Some simple explicit storage management for cons cells
+
+(def-standard-initial-binding *cons-pool* (%cons-pool nil))
+
+(defun cheap-cons (car cdr)
+  (let* ((pool *cons-pool*)
+         (cons (pool.data pool)))
+    (if cons
+      (locally (declare (type cons cons))
+        (setf (pool.data pool) (cdr cons)
+              (car cons) car
+              (cdr cons) cdr)
+        cons)
+      (cons car cdr))))
+
+(defun free-cons (cons)
+  (when (consp cons)
+    (locally (declare (type cons cons))
+      (setf (car cons) nil
+            (cdr cons) nil)
+      (let* ((pool *cons-pool*)
+             (freelist (pool.data pool)))
+        (setf (pool.data pool) cons
+              (cdr cons) freelist)))))
+
+(defun cheap-copy-list (list)
+  (let ((l list)
+        res)
+    (loop
+      (when (atom l)
+        (return (nreconc res l)))
+      (setq res (cheap-cons (pop l) res)))))
+
+(defun cheap-list (&rest args)
+  (declare (dynamic-extent args))
+  (cheap-copy-list args))
+
+;;; Works for dotted lists
+(defun cheap-free-list (list)
+  (let ((l list)
+        next-l)
+    (loop
+      (setq next-l (cdr l))
+      (free-cons l)
+      (when (atom (setq l next-l))
+        (return)))))
+
+(defmacro pop-and-free (place)
+  (setq place (require-type place 'symbol))     ; all I need for now.
+  (let ((list (gensym))
+        (cdr (gensym)))
+    `(let* ((,list ,place)
+            (,cdr (cdr ,list)))
+       (prog1
+         (car ,list)
+         (setf ,place ,cdr)
+         (free-cons ,list)))))
+
+;;; Support for defresource & using-resource macros
+(defun make-resource (constructor &key destructor initializer)
+  (%cons-resource constructor destructor initializer))
+
+(defun allocate-resource (resource)
+  (setq resource (require-type resource 'resource))
+  (let ((pool (resource.pool resource))
+        res)
+    (without-interrupts
+     (let ((data (pool.data pool)))
+       (when data
+         (setf res (car data)
+               (pool.data pool) (cdr (the cons data)))
+         (free-cons data))))
+    (if res
+      (let ((initializer (resource.initializer resource)))
+        (when initializer
+          (funcall initializer res)))
+      (setq res (funcall (resource.constructor resource))))
+    res))
+
+(defun free-resource (resource instance)
+  (setq resource (require-type resource 'resource))
+  (let ((pool (resource.pool resource))
+        (destructor (resource.destructor resource)))
+    (when destructor
+      (funcall destructor instance))
+    (without-interrupts
+     (setf (pool.data pool)
+           (cheap-cons instance (pool.data pool)))))
+  resource)
+
+
+
 
 (defpackage "OS"
   (:nicknames "OPERATING-SYSTEM" 
