@@ -258,13 +258,12 @@
 (defvar *modules* nil
   "Holds list of names of modules that have been loaded thus far.
    The names are case sensitive strings.")
-(defvar *module-file-alist* nil
-  "Holds an alist of modules and associated pathnames known to the system.")
+
 
 
 
 (defparameter *eof-value* (cons nil nil))
-(defvar *interrupt-level*)  ;Initialized by kernel.
+
 (defvar *gc-event-status-bits*)         ; also initialized by kernel
 
 (defparameter *top-listener* nil)
@@ -286,7 +285,10 @@
 (defparameter *autoload-lisp-package* nil)   ; Make 'em suffer
 (defparameter *apropos-case-sensitive-p* nil)
 
-(defloadvar *total-gc-microseconds* (malloc (* 5 8)))
+(defloadvar *total-gc-microseconds* (let* ((p (malloc (* 5 8))))
+                                      (dotimes (i 10 p)
+                                        (setf (%get-long p (* i 4)) 0))))
+
 (defloadvar *total-bytes-freed* (let* ((p (malloc 8)))
                                   (setf (%get-long p 0) 0
                                         (%get-long p 4) 0)
