@@ -325,55 +325,87 @@
             (logandc1 (ash mask position) integer))))
 
 (defun deposit-field (value bytespec integer)
-  (logior (logandc1 bytespec integer) (logand bytespec value)))
+  (if (> bytespec 0)    
+    (logior (logandc1 bytespec integer) (logand bytespec value))
+    (progn
+      (require-type value 'integer)
+      (require-type integer 'integer))))
 
 ;;;;;;;;;;  Byte field functions ;;;;;;;;;;;;;;;;
 
+;;; Size = 0, position = 0 -> 0
+;;; size = 0, position > 0 -> -position
+;;; else ->  (ash (byte-mask size) position)
 (defun byte (size position)
-  (unless (and (integerp position) (not (minusp position))) (report-bad-arg position 'unsigned-byte))
-  (ash (byte-mask size) position))
+  (unless (and (typep size 'integer)
+	       (>= size 0))
+    (report-bad-arg size 'unsigned-byte))
+  (unless (and (typep position 'integer)
+	       (>= position 0))
+    (report-bad-arg position 'unsigned-byte))
+  (if (eql 0 size)
+    (if (eql 0 position)
+      0
+      (- position))
+    (ash (byte-mask size) position)))
 
 
 
-(defun byte-size (bytespec) (logcount bytespec))
+(defun byte-size (bytespec)
+  (if (> bytespec 0)
+    (logcount bytespec)
+    0))
 
 (defun ldb (bytespec integer)
-  (if (and (fixnump bytespec) (fixnump integer))
+  (if (and (fixnump bytespec) (> (the fixnum bytespec) 0)  (fixnump integer))
     (%ilsr (byte-position bytespec) (%ilogand bytespec integer))
     (let ((size (byte-size bytespec))
           (position (byte-position bytespec)))
-      (if (and (bignump integer)
-               (<= size  (- 31 arch::fixnumshift))
-               (fixnump position))
-        (%ldb-fixnum-from-bignum integer size position)
-        (ash (logand bytespec integer) (- position))))))
+      (if (eql size 0)
+	(progn
+	  (require-type integer 'integer)
+	  0)
+	(if (and (bignump integer)
+		 (<= size  (- 31 ppc32::fixnumshift))
+		 (fixnump position))
+	  (%ldb-fixnum-from-bignum integer size position)
+	  (ash (logand bytespec integer) (- position)))))))
 
 (defun mask-field (bytespec integer)
-  (logand bytespec integer))
+  (if (>= bytespec 0)
+    (logand bytespec integer)
+    (logand integer 0)))
 
 (defun dpb (value bytespec integer)
-  (if (and (fixnump value) (fixnump bytespec) (fixnump integer))
+  (if (and (fixnump value)
+	   (fixnump bytespec)
+	   (> (the fixnum bytespec) 0)
+	   (fixnump integer))
     (%ilogior (%ilogand bytespec (%ilsl (byte-position bytespec) value))
               (%ilogand (%ilognot bytespec) integer))
     (deposit-field (ash value (byte-position bytespec)) bytespec integer)))
 
 (defun ldb-test (bytespec integer)
-  (logtest bytespec integer))
+  (if (> bytespec 0)
+    (logtest bytespec integer)
+    (progn
+      (require-type integer 'integer)
+      nil)))
 
 ; random associated stuff except for the print-object method which is still in
 ; "lib;numbers.lisp"
-(defun random-state (seed-1 seed-2)
+(defun initialize-random-state (seed-1 seed-2)
   (unless (and (fixnump seed-1) (%i<= 0 seed-1) (%i< seed-1 #x10000))
     (report-bad-arg seed-1 '(unsigned-byte 16)))
   (unless (and (fixnump seed-2) (%i<= 0 seed-2) (%i< seed-2 #x10000))
     (report-bad-arg seed-2 '(unsigned-byte 16)))
-  (let ((shift (%i- 16 arch::fixnum-shift)))
+  (let ((shift (%i- 16 ppc32::fixnum-shift)))
     (gvector :istruct
              'random-state
              (%ilsl shift seed-1)
              (%ilsl shift seed-2))))
 
-(defparameter *random-state* (random-state #xFBF1 9))
+(defparameter *random-state* (initialize-random-state #xFBF1 9))
 
 (defun make-random-state (&optional state &aux (seed-1 0) (seed-2 0))
   (if (eq state t)
@@ -392,15 +424,15 @@
   (declare (double-float b e result))
   (with-stack-double-floats ((temp))
     (%setf-double-float temp (#_pow b e))
-    (%df-check-exception-2 'expt b e (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-2 'expt b e (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-expt! (b e result)
   (declare (single-float b e result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float temp (#_powf b e))
-    (%sf-check-exception-2 'expt b e (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-2 'expt b e (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -417,15 +449,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_sin n))
-    (%df-check-exception-1 'sin n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'sin n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-sin! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_sinf n))
-    (%sf-check-exception-1 'sin n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'sin n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -440,15 +472,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_cos n))
-    (%df-check-exception-1 'cos n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'cos n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-cos! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_cosf n))
-    (%sf-check-exception-1 'cos n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'cos n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -463,15 +495,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_acos n))
-    (%df-check-exception-1 'acos n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'acos n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-acos! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_acosf n))
-    (%sf-check-exception-1 'acos n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'acos n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -486,15 +518,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_asin n))
-    (%df-check-exception-1 'asin n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'asin n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-asin! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_asinf n))
-    (%sf-check-exception-1 'asin n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'asin n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -509,15 +541,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_cosh n))
-    (%df-check-exception-1 'cosh n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'cosh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-cosh! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_coshf n))
-    (%sf-check-exception-1 'cosh n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'cosh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -532,15 +564,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_log n))
-    (%df-check-exception-1 'log n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'log n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-log! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_logf n))
-    (%sf-check-exception-1 'log n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'log n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -555,15 +587,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_tan n))
-    (%df-check-exception-1 'tan n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'tan n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-tan! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_tanf n))
-    (%sf-check-exception-1 'tan n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'tan n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -578,16 +610,16 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_atan n))
-    (%df-check-exception-1 'atan n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'atan n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 
 #-darwinppc-target
 (defun %single-float-atan! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_atanf n))
-    (%sf-check-exception-1 'atan n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'atan n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -603,15 +635,15 @@
   (declare (double-float x y result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_atan2 x y))
-    (%df-check-exception-2 'atan2 x y (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-2 'atan2 x y (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-atan2! (x y result)
   (declare (single-float x y result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_atan2f x y))
-    (%sf-check-exception-2 'atan2 x y (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-2 'atan2 x y (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -627,15 +659,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_exp n))
-    (%df-check-exception-1 'exp n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'exp n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-exp! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_expf n))
-    (%sf-check-exception-1 'exp n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'exp n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -650,15 +682,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_sinh n))
-    (%df-check-exception-1 'sinh n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'sinh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-sinh! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_sinhf n))
-    (%sf-check-exception-1 'sinh n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'sinh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -675,16 +707,16 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_tanh n))
-    (%df-check-exception-1 'tanh n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'tanh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 
 #-darwinppc-target
 (defun %single-float-tanh! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_tanhf n))
-    (%sf-check-exception-1 'tanh n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'tanh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -700,15 +732,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_asinh n))
-    (%df-check-exception-1 'asinh n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'asinh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-asinh! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_asinhf n))
-    (%sf-check-exception-1 'asinh n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'asinh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -724,15 +756,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_acosh n))
-    (%df-check-exception-1 'acosh n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'acosh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-acosh! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_acoshf n))
-    (%sf-check-exception-1 'acosh n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'acosh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -747,15 +779,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_atanh n))
-    (%df-check-exception-1 'atanh n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'atanh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-atanh! (n result)
   (declare (single-float n result)) 
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_atanhf n))
-    (%sf-check-exception-1 'atanh n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'atanh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
@@ -770,15 +802,15 @@
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
     (%setf-double-float TEMP (#_sqrt n))
-    (%df-check-exception-1 'sqrt n (%get-kernel-global 'ffi-exception))
+    (%df-check-exception-1 'sqrt n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
 #-darwinppc-target
 (defun %single-float-sqrt! (n result)
   (declare (single-float n result))
-  (with-stack-short-floats ((temp))
+  (ppc32::with-stack-short-floats ((temp))
     (%setf-short-float TEMP (#_sqrtf n))
-    (%sf-check-exception-1 'sqrt n (%get-kernel-global 'ffi-exception))
+    (%sf-check-exception-1 'sqrt n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
 #+darwinppc-target
