@@ -21,6 +21,48 @@
   (require "DARWIN-SYSCALLS"))
 
 
+(defun nanoseconds (n)
+  (check-type n (real 0 #xffffffff))
+  (multiple-value-bind (q r)
+      (floor n)
+    (if (zerop r)
+      (setq r 0)
+      (setq r (floor (* r 1000000000))))
+    (values q r)))
+  
+(defun semaphore-value (s)
+  (if (istruct-typep s 'semaphore)
+    (semaphore.value s)
+    (semaphore-value (require-type s 'semaphore))))
+
+(defun %wait-on-semaphore-ptr (s seconds nanoseconds)
+   (zerop
+    (the fixnum (ff-call
+		 (%kernel-import ppc32::kernel-import-wait-on-semaphore)
+		 :address s
+		 :unsigned seconds
+		 :unsigned nanoseconds
+		 :signed))))
+
+(defun %timed-wait-on-semaphore-ptr (s seconds nanoseconds)
+  (process-wait "semaphore wait" #'%wait-on-semaphore-ptr s seconds nanoseconds))
+  
+(defun wait-on-semaphore (s)
+  (%timed-wait-on-semaphore-ptr (semaphore-value s) 1 0))
+
+(defun timed-wait-on-semaphore (s duration)
+  (multiple-value-bind (secs nanos) (nanoseconds duration)
+    (%wait-on-semaphore-ptr (semaphore-value s) secs nanos)))
+
+(defun %signal-semaphore-ptr (p)
+  (ff-call
+   (%kernel-import ppc32::kernel-import-signal-semaphore)
+   :address p
+   :signed-fullword))
+
+(defun signal-semaphore (s)
+  (%signal-semaphore-ptr (semaphore-value s)))
+
 (defun %os-getcwd (buf bufsize)
   ;; Return N < 0, if error
   ;;        N < bufsize: success, string is of length n
@@ -77,6 +119,18 @@
 	(%get-cstring env-ptr))))
   )
 
+(defun setenv (key value &optional (overwrite t))
+  (with-cstrs ((ckey key)
+	       (cvalue value))
+    (#_setenv ckey cvalue (if overwrite 1 0))))
+
+(defun setuid (uid)
+  (syscall os::setuid uid))
+
+(defun setgid (uid)
+  (syscall os::setgid uid))
+  
+
 ;;; On Linux, "stat" & friends are implemented in terms of deeper,
 ;;; darker things that need to know what version of the stat buffer
 ;;; they're talking about.
@@ -88,7 +142,7 @@
        (pref stat :stat.st_mode)
        (pref stat :stat.st_size)
        #+linuxppc-target
-       (pref stat :stat.st_mtime)
+       (pref stat :stat.st_mtim.tv_sec)
        #+darwinppc-target
        (pref stat :stat.st_mtimespec.tv_sec)
        (pref stat :stat.st_ino)
@@ -144,14 +198,16 @@
 	    ((eql kind #$S_IFLNK) :link)
 	    ((eql kind #$S_IFIFO) :pipe)
 	    ((eql kind #$S_IFSOCK) :socket)
-	    ((eql kind #$S_IFCHR) :tty)
+	    ((eql kind #$S_IFCHR) :character-special)
 	    (t :special)))))
 
 (defun %unix-file-kind (path &optional check-for-link)
   (%file-kind (nth-value 1 (%stat path check-for-link))))
 
 (defun %unix-fd-kind (fd)
-  (%file-kind (nth-value 1 (%fstat fd))))
+  (if (isatty fd)
+    :tty
+    (%file-kind (nth-value 1 (%fstat fd)))))
 
 (defun %uts-string (result idx buf)
   (if (eql 0 result)
@@ -217,10 +273,6 @@
 (defun timeval->milliseconds (tv)
     (+ (* 1000 (pref tv :timeval.tv_sec)) (round (pref tv :timeval.tv_usec) 1000)))
 
-(defun get-internal-real-time ()
-  (rlet ((tv :timeval))
-    (#_gettimeofday tv (%null-ptr))
-    (timeval->milliseconds tv)))
 
 (defun %add-timevals (result a b)
   (let* ((seconds (+ (pref a :timeval.tv_sec) (pref b :timeval.tv_sec)))
@@ -231,25 +283,21 @@
 	  (pref result :timeval.tv_usec) micros)
     result))
 
-(defun %sub-timevals (result a b)
-  (let* ((seconds (- (pref a :timeval.tv_sec) (pref b :timeval.tv_sec)))
-	 (micros (- (pref a :timeval.tv_usec) (pref b :timeval.tv_usec))))
-    (if (< micros 0)
-      (setq seconds (1- seconds) micros (+ micros 1000000)))
-    (setf (pref result :timeval.tv_sec) seconds
-	  (pref result :timeval.tv_usec) micros)
-    result))
 
+
+
+#-darwinppc-target
 (defun %%rusage (usage &optional (who #$RUSAGE_SELF))
   (syscall os::getrusage who usage))
+#+darwinppc-target
+(defun %%rusage (usage &optional (who #$RUSAGE_SELF))
+  (syscall os::getrusage who usage)
+  (rlet ((count :natural_t #$TASK_THREAD_TIMES_INFO_COUNT))
+    (#_task_info (#_mach_task_self) #$TASK_THREAD_TIMES_INFO usage count)))
+    
+
+
 
-(defun get-internal-run-time ()
-  (rlet ((usage :rusage)
-	 (total :timeval))
-    (%%rusage usage)
-    (timeval->milliseconds (%add-timevals total 
-					  (pref usage :rusage.ru_utime) 
-					  (pref usage :rusage.ru_stime)))))
 
 
 
@@ -269,6 +317,14 @@
           (without-interrupts
            (%get-cstring (pref pw :passwd.pw_name))))))))
 
+(defun %utimes (namestring)
+  (with-cstrs ((cnamestring namestring))
+    (let* ((err (#_utimes cnamestring (%null-ptr))))
+      (declare (fixnum err))
+      (or (eql err 0)
+          (%errno-disp err namestring)))))
+         
+
 (defun isatty (fd)
   (= 1 (#_isatty fd)))
 
@@ -340,6 +396,7 @@
 	    (setf (shlib.pathname lib) nil
 	      (shlib.base lib) nil
 	      (shlib.map lib) nil)
+            (unload-foreign-variables lib)
 	    (unload-library-entrypoints lib)))))))
 
 #+darwinppc-target
@@ -442,6 +499,7 @@
       (%errno-disp ans nil)
       (values (ldb (byte 32 32) ans) (ldb (byte 32 0) ans)))))
 
+
 (defstruct external-process
   pid
   %status
@@ -454,11 +512,15 @@
   plist
   token
   core
+  args
+  (signal (make-semaphore))
   )
 
 (defmethod print-object ((p external-process) stream)
   (print-unreadable-object (p stream :type t :identity t)
     (let* ((status (external-process-%status p)))
+      (let* ((*print-length* 3))
+	(format stream "~a" (external-process-args p)))
       (format stream "[~d] (~a" (external-process-pid p) status)
       (unless (eq status :running)
 	(format stream " : ~d" (external-process-%exit-code p)))
@@ -541,39 +603,113 @@
 		  (cons write-pipe close-in-parent)
 		  (cons read-pipe close-on-error))))))))
 
-(defloadvar *external-processes* nil)
-(defloadvar *watched-fd-handlers* nil)
+(let* ((external-processes ())
+       (watched-fd-handlers ())
+       (external-processes-lock (make-lock))
+       (watched-fd-handlers-lock (make-lock)))
+  (defun add-external-process (p)
+    (with-lock-grabbed (external-processes-lock)
+      (push p external-processes)))
+  (defun remove-external-process (p)
+    (with-lock-grabbed (external-processes-lock)
+      (setq external-processes (delete p external-processes))))
+  (defun add-watched-fd-handler (h)
+    (with-lock-grabbed (watched-fd-handlers-lock)
+      (push h watched-fd-handlers)))
+  (defun remove-watched-fd-handler (h)
+    (with-lock-grabbed (watched-fd-handlers-lock)
+      (setq watched-fd-handlers (delete h watched-fd-handlers))))
+  (defun watch-file-descriptors ()
+    (with-lock-grabbed (watched-fd-handlers-lock)
+      (dolist (h watched-fd-handlers) (funcall h))
+      (null watched-fd-handlers)))
+  (defun check-all-pids ()
+    (with-lock-grabbed (external-processes-lock)
+      (dolist (p external-processes)
+	(let* ((statusflags (check-pid (external-process-pid p)))
+	       (oldstatus (external-process-%status p)))
+	  (cond ((null statusflags)
+		 (remove-external-process p))
+		((eq statusflags t))	; Running.
+		(t
+		 (multiple-value-bind (status code core)
+		     (cond ((wifstopped statusflags)
+			    (values :stopped (wstopsig statusflags)))
+			   ((wifexited statusflags)
+			    (values :exited (wexitstatus statusflags)))
+			   (t
+			    (let* ((signal (wtermsig statusflags)))
+			      (declare (fixnum signal))
+			      (values
+			       (if (or (= signal #$SIGSTOP)
+				       (= signal #$SIGTSTP)
+				       (= signal #$SIGTTIN)
+				       (= signal #$SIGTTOU))
+				 :stopped
+				 :signaled)
+			       signal
+			       (logtest #$WCOREFLAG statusflags)))))
+		   (setf (external-process-%status p) status
+			 (external-process-%exit-code p) code
+			 (external-process-core p) core)
+		   (let* ((status-hook (external-process-status-hook p)))
+		     (when (and status-hook (not (eq oldstatus status)))
+		       (funcall status-hook p)))
+		   (when (or (eq status :exited)
+			     (eq status :signaled))
+		     (remove-external-process p)))))))
+      (null external-processes)))
+  ;; Returns a copy, for debugging.
+  (defun watched-fd-handlers ()
+    (with-lock-grabbed (watched-fd-handlers-lock)
+      (copy-list watched-fd-handlers)))
+  ;; Likewise
+  (defun external-processes ()
+    (with-lock-grabbed (external-processes-lock)
+      (copy-list external-processes)))
+  )
 
-(defvar *external-process-watchdog*)
-
-(defun watch-file-descriptors ()
-  (dolist (handler *watched-fd-handlers*) (funcall handler)))
 
 
 (defun watch-fd-output (in-fd out-stream token)
   (incf (car token))
-  (let* ((handler nil))
+  (let* ((handler ()))
     (setq handler
 	  #'(lambda ()
 	      (when (fd-input-available-p in-fd 0)
 		(%stack-block ((buf 1024))
-	          (let* ((n (fd-read in-fd buf 1024)))
+		  (let* ((n (fd-read in-fd buf 1024)))
 		    (declare (fixnum n))
 		    (if (<= n 0)
 		      (progn
+			(remove-watched-fd-handler handler)
 			(without-interrupts
-			 (setq *watched-fd-handlers*
-			       (delete handler *watched-fd-handlers*))
 			 (decf (car token))
 			 (fd-close in-fd)))
 		      (let* ((string (make-string 1024)))
 			(declare (dynamic-extent string))
 			(%copy-ptr-to-ivector buf 0 string 0 n)
 			(write-sequence string out-stream :end n))))))))
-    (push handler *watched-fd-handlers*)
-    nil))
-    
-		    
+    (add-watched-fd-handler handler))
+  nil)
+  
+(defun run-external-process (proc in-fd out-fd error-fd)
+  (call-with-string-vector
+   #'(lambda (argv)
+       (let* ((child-pid (#_fork)))
+	 (declare (fixnum child-pid))
+	 (cond ((zerop child-pid)
+		;; Running in the child; do an exec
+		(without-interrupts
+		 (exec-with-io-redirection
+		  in-fd out-fd error-fd argv)))
+	       ((> child-pid 0)
+		;; Running in the parent: success
+		(setf (external-process-pid proc) child-pid)
+		(signal-semaphore (external-process-signal proc))
+		(add-external-process proc)))))
+   (external-process-args proc)))
+
 		
 (defun run-program (program args &key
 			    (wait t) pty
@@ -610,36 +746,26 @@
 	       (get-descriptor-for error token close-in-parent close-on-error
 				   :direction :output
 				   :if-exists if-error-exists)))
-	   (call-with-string-vector
-	    #'(lambda (argv)
-		(let* ((child-pid (#_fork)))
-		  (declare (fixnum child-pid))
-		  (cond ((zerop child-pid)
-			 ;; Running in the child; do an exec
-			 (without-interrupts
-			  (exec-with-io-redirection
-			   in-fd out-fd error-fd argv)))
-			((> child-pid 0)
-			 ;; Running in the parent: success
-			 (setq proc
-			       (make-external-process
-				:pid child-pid
-				:%status :running
-				:input in-stream
-				:output out-stream
-				:error error-stream
-				:token token
-				:status-hook status-hook))
-			 (push proc *external-processes*)))))
-	    args))
-      (unless (process-active-p *external-process-watchdog*)
-	(process-reset-and-enable *external-process-watchdog*))
+	   (setq proc
+		 (make-external-process
+		  :pid nil
+		  :args args
+		  :%status :running
+		  :input in-stream
+		  :output out-stream
+		  :error error-stream
+		  :token token
+		  :status-hook status-hook))
+	   (process-interrupt *initial-process* #'run-external-process proc in-fd out-fd error-fd)
+	   (wait-on-semaphore (external-process-signal proc))
+	   )
+
       (dolist (fd close-in-parent) (fd-close fd))
-      (unless proc
+      (unless (external-process-pid proc)
 	(dolist (fd close-on-error) (fd-close fd)))
-      (when (and wait proc)
+      (when (and wait (external-process-pid proc))
 	(external-process-wait proc)))
-    proc))
+    (and proc (external-process-pid proc) proc)))
 
 #|
        WIFEXITED(status)
@@ -695,7 +821,7 @@
       (and (not (wifstopped ,statname)) (not (wifexited ,statname))))))
 
 
-(defun check-pid (pid &optional (flags (logior #$WNOHANG #$WUNTRACED)))
+(defun check-pid (pid &optional (flags (logior  #$WNOHANG #$WUNTRACED)))
   (declare (fixnum pid))
   (rlet ((status :signed))
     (let* ((retval (#_waitpid pid status flags)))
@@ -705,43 +831,6 @@
 	(zerop retval)))))
 
 
-(defun check-all-pids ()
-  (dolist (p *external-processes*)
-    (let* ((statusflags (check-pid (external-process-pid p)))
-	   (oldstatus (external-process-%status p)))
-      (cond ((null statusflags)
-	     (without-interrupts
-	      (setq *external-processes* (delete p *external-processes*))))
-	    ((eq statusflags t))	; Running.
-	    (t
-	     (multiple-value-bind (status code core)
-		 (cond ((wifstopped statusflags)
-			(values :stopped (wstopsig statusflags)))
-		       ((wifexited statusflags)
-			(values :exited (wexitstatus statusflags)))
-		       (t
-			(let* ((signal (wtermsig statusflags)))
-			  (declare (fixnum signal))
-			  (values
-			   (if (or (= signal #$SIGSTOP)
-				   (= signal #$SIGTSTP)
-				   (= signal #$SIGTTIN)
-				   (= signal #$SIGTTOU))
-			     :stopped
-			     :signaled)
-			   signal
-			   (logtest #$WCOREFLAG statusflags)))))
-	       (setf (external-process-%status p) status
-		     (external-process-%exit-code p) code
-		     (external-process-core p) core)
-	       (let* ((status-hook (external-process-status-hook p)))
-		 (when (and status-hook (not (eq oldstatus status)))
-		   (funcall status-hook p)))
-	       (when (or (eq status :exited)
-			 (eq status :signaled))
-		 (without-interrupts
-		  (setq *external-processes*
-			(delete p *external-processes*))))))))))
 
 
 
@@ -790,3 +879,100 @@
   (case (%unix-fd-kind fd)
     (:tty t)
     (t nil)))
+
+
+(defstruct (shared-resource (:constructor make-shared-resource (name)))
+  (name)
+  (lock (make-lock))
+  (primary-owner *current-process*)
+  (primary-owner-notify (make-semaphore))
+  (current-owner nil)
+  (requestors (make-dll-header)))
+
+(defstruct (shared-resource-request
+	     (:constructor make-shared-resource-request (process))
+	     (:include dll-node))
+  process
+  (signal (make-semaphore)))
+	     
+
+;; Returns NIL if already owned by calling thread, T otherwise
+(defun %acquire-shared-resource (resource  &optional verbose)
+  (let* ((current *current-process*))
+    (with-lock-grabbed ((shared-resource-lock resource))
+      (let* ((secondary (shared-resource-current-owner resource)))
+	(if (or (eq current secondary)
+		(and (null secondary)
+		     (eq current (shared-resource-primary-owner resource))))
+	  (return-from %acquire-shared-resource nil))))
+    (let* ((request (make-shared-resource-request *current-process*)))
+      (when verbose
+	(format t "~%~%;;;~%;;; ~a requires access to ~a~%;;;~%~%"
+		*current-process* (shared-resource-name resource)))
+      (with-lock-grabbed ((shared-resource-lock resource))
+	(append-dll-node request (shared-resource-requestors resource)))
+      (wait-on-semaphore (shared-resource-request-signal request))
+      (assert (eq current (shared-resource-current-owner resource)))
+      (when verbose
+	(format t "~%~%;;;~%;;; ~a is now owned by ~a~%;;;~%~%"
+		(shared-resource-name resource) current))
+      t)))
+
+;;; If we're the primary owner and there is no secondary owner, do nothing.
+;;; If we're the secondary owner, cease being the secondary owner.
+(defun %release-shared-resource (r)
+  (let* ((not-any-owner ()))
+    (with-lock-grabbed ((shared-resource-lock r))
+      (let* ((current *current-process*)
+	     (primary (shared-resource-primary-owner r))
+	     (secondary (shared-resource-current-owner r)))
+	(unless (setq not-any-owner
+		      (not (or (eq current secondary)
+                               (and (null secondary)
+                                    (eq current primary)))))
+	  (when (eq current secondary)
+	    (setf (shared-resource-current-owner r) nil)
+	    (signal-semaphore (shared-resource-primary-owner-notify r))))))
+    (when not-any-owner
+      (signal-program-error "Process ~a does not own ~a" *current-process*
+			    (shared-resource-name r)))))
+
+;;; The current thread should be the primary owner; there should be
+;;; no secondary owner.  Wakeup the specified (or first) requesting
+;;; process, then block on our semaphore 
+(defun %yield-shared-resource (r &optional to)
+  (let* ((request nil))
+    (with-lock-grabbed ((shared-resource-lock r))
+      (let* ((current *current-process*)
+	     (primary (shared-resource-primary-owner r)))
+	(when (and (eq current primary)
+		   (null (shared-resource-current-owner r)))
+	  (setq request
+		(let* ((header (shared-resource-requestors r)))
+		  (if to 
+		    (do-dll-nodes (node header)
+		      (when (eq to (shared-resource-request-process node))
+			(return node)))
+		    (let* ((first (dll-header-first header)))
+		      (unless (eq first header)
+			first)))))
+	  (when request
+	    (remove-dll-node request)
+            (setf (shared-resource-current-owner r)
+                  (shared-resource-request-process request))
+	    (signal-semaphore (shared-resource-request-signal request))))))
+    (when request
+      (wait-on-semaphore (shared-resource-primary-owner-notify r))
+      (format t "~%;;;~%;;;control of ~a restored to ~a~%;;;~&"
+	      (shared-resource-name r)
+	      *current-process*))))
+
+
+      
+
+(defun %shared-resource-requestor-p (r proc)
+  (with-lock-grabbed ((shared-resource-lock r))
+    (do-dll-nodes (node (shared-resource-requestors r))
+      (when (eq proc (shared-resource-request-process node))
+	(return t)))))
+
