@@ -290,7 +290,7 @@
 (defmethod print-object ((rs random-state) stream)
   ;;Besides #.grossness, this is buggy because cl:random-state
   ;;is not allowed to have a function definition...
-  (let ((shift (%i- 16 arch::fixnum-shift)))
+  (let ((shift (%i- 16 ppc32::fixnum-shift)))
     (format stream "#.(~S ~S ~S)"   ;>> #.GAG!!!
             'random-state
             (%ilsr shift (%svref rs 1))
@@ -334,9 +334,16 @@
             (setq n0 (gcd-2 n0 (%lexpr-ref numbers count i)))))))))
 
 (defun lcm-2 (n0 n1)
-  (let* ((small (if (< n0 n1) n0 n1))
-         (large (if (eq small n0) n1 n0)))
-    (* (truncate large (gcd n0 n1)) small)))
+  (or (typep n0 'integer) (report-bad-arg n0 'integer))
+  (or (typep n1 'integer) (report-bad-arg n1 'integer))
+  (locally (declare (integer n0 n1))
+    (if (zerop n0)
+      0
+      (if (zerop n1)
+	0
+	(let* ((small (if (< n0 n1) n0 n1))
+	       (large (if (eq small n0) n1 n0)))
+	  (values (truncate (abs (* n0 n1)) (gcd large small))))))))
 
 (defun lcm (&lexpr numbers)
   (let* ((count (%lexpr-count numbers)))
@@ -346,32 +353,41 @@
       (let* ((n0 (%lexpr-ref numbers count 0)))
         (if (= count 1)
           (%integer-abs n0)
-          (do* ((i 1 (1+ i)))
-               ((= i count) n0)
-            (declare (fixnum i))
-            (setq n0 (lcm-2 n0 (%lexpr-ref numbers count i)))))))))
+	  (if (= count 2)
+	    (lcm-2 n0 (%lexpr-ref numbers count 1))
+	    (do* ((i 1 (1+ i)))
+		 ((= i count) n0)
+	      (declare (fixnum i))
+	      (setq n0 (lcm-2 n0 (%lexpr-ref numbers count i))))))))))
 
 
 #|
-(defun rationalize (number)
-  (cond ((not (floatp number)) (rational number))
-        ((minusp number) (- (rationalize (- number))))
-        ((zerop number) 0)
-        (t (let ((onum 1) (oden 0) num (den 1) rem)
-             (multiple-value-setq (num rem) (truncate number))
-             (until (<= (abs (/ (- number (/ (float num) den)) number))
-                        single-float-epsilon)
-               (multiple-value-bind (q r) (truncate 1.0 rem)
-                 (setq rem (/ r rem))
-                 (let ((nnum (+ (* q num) onum)))
-                   (setq onum num num nnum))
-                 (let ((nden (+ (* q den) oden)))
-                   (setq oden den den nden))))
-             (/ num den)))))
-;Rationalize failed. Input 1.9998229581607005 Rational 70903515/35454896 Float 1.9998229581607008
-; also gets overflow and underflow sometimes
+(defun rationalize (x)
+  (etypecase x
+    (rational x)
+    (real
+     (cond ((minusp x) (- (rationalize (- x))))
+	   ((zerop x) 0)
+	   (t
+	    (let ((eps (etypecase x
+			 (single-float single-float-epsilon)
+			 (double-float double-float-epsilon)))
+		  (y ())
+		  (a ()))
+	      (do ((xx x (setq y (/ (float 1.0 x) (- xx (float a x)))))
+		   (num (setq a (truncate x))
+			(+ (* (setq a (truncate y)) num) onum))
+		   (den 1 (+ (* a den) oden))
+		   (onum 1 num)
+		   (oden 0 den))
+		  ((and (not (zerop den))
+			(not (> (abs (/ (- x (/ (float num x)
+						(float den x)))
+					x))
+				eps)))
+		   (integer-/-integer num den)))))))))
 |#
-; Kalman's more better one
+
 (defun rationalize (number)
   (if (floatp number)
     (labels ((simpler-rational (less-predicate lonum loden hinum hiden
@@ -388,13 +404,13 @@
                                      loden (- lonum (* (1- term) loden)))
                    (values (+ den (* (1- term) num)) num)))))                           
       (multiple-value-bind (fraction exponent sign) (integer-decode-float number)
-        ; the first 2 tests may be unnecessary - I think the check for denormalized
-        ; is compensating for a bug in 3.0 re floating a rational (in order to pass tests in ppc-test-arith).
+        ;; the first 2 tests may be unnecessary - I think the check
+        ;; for denormalized is compensating for a bug in 3.0 re
+        ;; floating a rational (in order to pass tests in
+        ;; ppc-test-arith).
         (if (or (and (typep number 'double-float)  ; is it denormalized
                      (eq exponent #.(nth-value 1 (integer-decode-float least-positive-double-float)))) ; aka -1074))
-                (and nil ;(typep number 'short-float) ; was needed to pass tests but bug was elsewhere
-                     ; this won't cross compile but I assume we don't worry about that any more.
-                     (eq exponent #.(nth-value 1 (integer-decode-float least-positive-short-float)))) ; aka -149))
+                (eq exponent #.(nth-value 1 (integer-decode-float least-positive-short-float))) ; aka -149))
                 (zerop (logand fraction (1- fraction)))) ; or a power of two
           (rational number)
           (if (minusp exponent)
@@ -516,8 +532,10 @@
                (locally (declare (fixnum op))
                  (and (>= op 0)
                       (<= op 15))))
-    (report-bad-arg op '(integer 0 16)))
-  (funcall (%svref *boole-ops* op) integer1 integer2))
+    (report-bad-arg op '(integer 0 15)))
+  (funcall (%svref *boole-ops* op)
+	   (require-type integer1 'integer)
+	   (require-type integer2 'integer)))
 
 
 (defun %integer-power (b e)
@@ -534,7 +552,7 @@
         (setq b (* b b) e next)))))
 
 (defun signum (x)
-  (cond ((complexp x) (cis (phase x)))
+  (cond ((complexp x) (if (zerop x) x (/ x (abs x))))
         ((rationalp x) (if (plusp x) 1 (if (zerop x) 0 -1)))
         ((zerop x) (float 0.0 x))
         (t (float-sign x))))
@@ -572,7 +590,7 @@
     (/ (- (exp x) (exp (- x))) 2)
     (if (typep x 'double-float)
       (%double-float-sinh! x (%make-dfloat))
-      (with-stack-short-floats ((sx x))
+      (ppc32::with-stack-short-floats ((sx x))
 	(%single-float-sinh! sx (%make-sfloat))))))
 
 
@@ -581,7 +599,7 @@
     (/ (+ (exp x) (exp (- x))) 2)
     (if (typep x 'double-float)
       (%double-float-cosh! x (%make-dfloat))
-      (with-stack-short-floats ((sx x))
+      (ppc32::with-stack-short-floats ((sx x))
 	(%single-float-cosh! sx (%make-sfloat))))))
 
 (defun tanh (x)
@@ -589,7 +607,7 @@
     (/ (sinh x) (cosh x))
     (if (typep x 'double-float)
       (%double-float-tanh! x (%make-dfloat))
-      (with-stack-short-floats ((sx x))
+      (ppc32::with-stack-short-floats ((sx x))
 	(%single-float-tanh! sx (%make-sfloat))))))
 
 (defun asinh (x)
@@ -597,14 +615,14 @@
     (log (+ x (sqrt (+ 1 (* x x)))))
     (if (typep x 'double-float)
       (%double-float-asinh! x (%make-dfloat))
-      (with-stack-short-floats ((sx x))
+      (ppc32::with-stack-short-floats ((sx x))
 	(%single-float-asinh! sx (%make-sfloat))))))
 
 (defun acosh (x)
   (if (and (realp x) (<= 1.0 x))
     (if (typep x 'double-float)
       (%double-float-acosh! x (%make-dfloat))
-      (with-stack-short-floats ((sx x))
+      (ppc32::with-stack-short-floats ((sx x))
 	(%single-float-acosh! sx (%make-sfloat))))
     (* 2 (log (+ (sqrt (/ (1+ x) 2)) (sqrt (/ (1- x) 2)))))))
 
